#ifndef __PHYSICSTEST_H__
#define __PHYSICSTEST_H__

#include <FApp.h>
#include <FBase.h>
#include <FGraphics.h>
#include <FSystem.h>
#include "TimerThread.h"
#include "chipmunk/chipmunk.h"

/**
 * [PhysicsTest] application must inherit from Application class
 * which provides basic features necessary to define an application.
 */
class PhysicsTest :
	public Osp::App::Application,
	public Osp::System::IScreenEventListener,
	public ITimerThreadEventListener,
	public Osp::Base::Runtime::ITimerEventListener
{
public:
	// [PhysicsTest] application must have a factory method that creates an instance of itself.
	static Osp::App::Application* CreateInstance(void);

public:
	PhysicsTest();
	~PhysicsTest();

public:
	// Called when the application is initializing.
	bool OnAppInitializing(Osp::App::AppRegistry& appRegistry);

	// Called when the application is terminating.
	bool OnAppTerminating(Osp::App::AppRegistry& appRegistry, bool forcedTermination = false);

	// Called when the application's frame moves to the top of the screen.
	void OnForeground(void);

	// Called when this application's frame is moved from top of the screen to the background.
	void OnBackground(void);

	// Called when the system memory is not sufficient to run the application any further.
	void OnLowMemory(void);

	// Called when the battery level changes.
	void OnBatteryLevelChanged(Osp::System::BatteryLevel batteryLevel);

	// Called when the screen turns on.
	void OnScreenOn (void);

	// Called when the screen turns off.
	void OnScreenOff (void);

	void OnTimerExpired(Osp::Base::Runtime::Timer &timer);
	void OnTimerTick(TimerThread &thread);

private:
	void InitShapes();
	void StartTimers();
	void StopTimers();

private:
	TimerThread __physicsThread;
	cpSpace *__pSpace;
	Osp::Base::Runtime::Timer __redrawTimer;
	Osp::Base::Runtime::Mutex __mutex;
};

#endif	//__PHYSICSTEST_H__
