/**
 * Name        : PhysicsTest
 * Version     : 
 * Vendor      : 
 * Description : 
 */

#include "PhysicsTest.h"
#include "chipmunk/chipmunk.h"
#include <time.h>

using namespace Osp::App;
using namespace Osp::Base;
using namespace Osp::System;
using namespace Osp::Graphics;
using namespace Osp::Base::Runtime;

void DrawShapes(void *ptr, void *context);

PhysicsTest::PhysicsTest() : __physicsThread(*this, 1000/30), __pSpace(NULL)
{
}

PhysicsTest::~PhysicsTest()
{
	if (__pSpace)
	{
		cpSpaceFreeChildren(__pSpace);
		cpSpaceFree(__pSpace);
	}
}

Application*
PhysicsTest::CreateInstance(void)
{
	// Create the instance through the constructor.
	return new PhysicsTest();
}

bool
PhysicsTest::OnAppInitializing(AppRegistry& appRegistry)
{
	// TODO:
	// Initialize UI resources and application specific data.
	// The application's permanent data and context can be obtained from the appRegistry.
	//
	// If this method is successful, return true; otherwise, return false.
	// If this method returns false, the application will be terminated.

	cpInitChipmunk();
	__pSpace = cpSpaceNew();
	__physicsThread.Construct("PhysicsThread");
	__redrawTimer.Construct(*this);
	__mutex.Create();
	InitShapes();

	// Uncomment the following statement to listen to the screen on/off events.
	PowerManager::SetScreenEventListener(*this);
	return true;
}

bool
PhysicsTest::OnAppTerminating(AppRegistry& appRegistry, bool forcedTermination)
{
	// TODO:
	// Deallocate resources allocated by this application for termination.
	// The application's permanent data and context can be saved via appRegistry.
	StopTimers();
	return true;
}

void
PhysicsTest::OnForeground(void)
{
	StartTimers();
}

void
PhysicsTest::OnBackground(void)
{
	StopTimers();
}

void
PhysicsTest::OnLowMemory(void)
{
	// TODO:
	// Free unused resources or close the application.
}

void
PhysicsTest::OnBatteryLevelChanged(BatteryLevel batteryLevel)
{
	// TODO:
	// Handle any changes in battery level here.
	// Stop using multimedia features(camera, mp3 etc.) if the battery level is CRITICAL.
}

void
PhysicsTest::OnScreenOn (void)
{
	StartTimers();
}

void
PhysicsTest::OnScreenOff (void)
{
	StopTimers();
}

void
PhysicsTest::OnTimerTick(TimerThread &thread)
{
	__mutex.Acquire();
	cpSpaceStep(__pSpace, 1.0f / 30.0f);
	__mutex.Release();
}

void
PhysicsTest::InitShapes()
{
	const int balls[][2] = {{240, 400}, {120, 200}, {360, 200}, {180, 100}, {420, 100}};
	const int walls[][4] = {{0, 0, 480, 0}, {480, 0, 480, 800}, {480, 800, 0, 800}, {0, 800, 0, 0}};
	Utility::Math::Srand(time(NULL));

	cpBody *ballBody = NULL;
	cpShape *ballShape = NULL;

	for (unsigned int i = 0; i < 5; i++)
	{
		ballBody = cpBodyNew(10.0, INFINITY);
		ballBody->p = cpv(balls[i][0], balls[i][1]);
		ballShape = cpCircleShapeNew(ballBody, 40.0, cpv(0, 0));
		ballShape->e = 0.8;
		cpSpaceAddBody(__pSpace, ballBody);
		cpSpaceAddShape(__pSpace, ballShape);

		cpBodyApplyImpulse(ballBody, cpv(Utility::Math::Rand() % 4000 + 1500, Utility::Math::Rand() % 4000 + 1500), cpvzero);
	}

	cpBody *wallBody = NULL;
	cpShape *wallShape = NULL;

	for (unsigned int i = 0; i < 4; i++)
	{
		wallBody = cpBodyNew(INFINITY, INFINITY);
		wallBody->p = cpv(0, 0);
		wallShape = cpSegmentShapeNew(wallBody, cpv(walls[i][0], walls[i][1]), cpv(walls[i][2], walls[i][3]), 1);
		wallShape->collision_type = 1;
		wallShape->e = 1.0f;
		wallShape->u = 1.0f;
		cpSpaceAddStaticShape(__pSpace, wallShape);
	}
}

void
PhysicsTest::OnTimerExpired(Timer &timer)
{
	__mutex.Acquire();
	Canvas* pCanvas = GetAppFrame()->GetCanvasN();
	pCanvas->Clear();
	cpSpaceHashEach(__pSpace->activeShapes, DrawShapes, pCanvas);
	pCanvas->Show();
	delete pCanvas;
	__mutex.Release();
	__redrawTimer.Start(1000 / 60);
}

void
PhysicsTest::StartTimers()
{
	__physicsThread.Start();
	__redrawTimer.Start(1000 / 60);
}

void
PhysicsTest::StopTimers()
{
	__physicsThread.Stop();
	__redrawTimer.Cancel();
}

void
DrawShapes(void *ptr, void *context)
{
	Canvas *pCanvas = reinterpret_cast<Canvas *>(context);
	cpShape *pShape = reinterpret_cast<cpShape *>(ptr);

	if (pShape->collision_type == 0)
	{
		pCanvas->FillEllipse(Color::COLOR_BLUE, Rectangle(pShape->body->p.x - 40, pShape->body->p.y - 40, 80, 80));
	}
}
